/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include <gnome.h>

#include "interface.h"
#include "support.h"
#include "main.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/ioctl.h>
#include <linux/ppdev.h>

#include <unistd.h>
#include <sys/mman.h>
#include <linux/videodev.h>

#include <math.h>

#include <pthread.h>

#define IN_TV                   0
#define IN_COMPOSITE    1
#define IN_COMPOSITE2   2
#define IN_SVIDEO               3
#define IN_DEFAULT              8

#define NORM_PAL                0
#define NORM_NTSC               1
#define NORM_SECAM              2
#define NORM_DEFAULT    0

#define WIDTH 640
#define HEIGHT 480

#define source_pixel(X,Y) ((int)source[(Y)*640+(X)])
#define dest_pixel(X,Y) ((int)dest[(Y)*640+(X)])

int webcam_thread;

int fd_parport;
int fd_webcam;

int state, oldstate;

struct foo
{
    int a;
    int b;
    int c;
    int d;
    int e;
} in;

struct video_picture v_pict;        // used to store prefered webcam palette

guchar image1[WIDTH*HEIGHT];   // main 640x480 live display image buffer
guchar image2[WIDTH*HEIGHT];   // main 640x480 live display image buffer

guchar main_camera[WIDTH*HEIGHT];   // main 640x480 live display image buffer
guchar camera_1[320*240];
guchar camera_2[320*240];
guchar camera_3[320*240];
guchar camera_4[320*240];
guchar final_camera[WIDTH*HEIGHT];   // main 640x480 live display image buffer
guchar hough_camera[401*180];

long int hough_space[401*180];

double sintable[360], costable[360];

// hough statistics
int block[180][3], blocks = 0; /* start, stop, count */

// needed for lookup_widget() calls
GtkWidget *window1;

/* Prototypes */
void   pp_read(void);
void   pp_write( unsigned char a );
gint   pp_callback( gpointer data );

void   webcam_capture( void );
char * webcam_getimage( int *size );
gint   webcam_callback( gpointer data );


void pp_read(void)
{
    unsigned char data;
    ioctl (fd_parport, PPRSTATUS, &data);

    in.a = (data & 1<<7) ? 1 : 0;
    in.b = (data & 1<<6) ? 0 : 1;
    in.c = (data & 1<<5) ? 0 : 1;
    in.d = (data & 1<<4) ? 0 : 1;
    in.e = (data & 1<<3) ? 0 : 1;
}

void pp_write( unsigned char a )
{
    ioctl( fd_parport, PPWDATA, &a );
}

gint pp_callback( gpointer data )
{
    static int pressed = 0;

    pp_read();
    if( in.e )
    {
        if( !pressed )
        {
            pressed = 1;
            state = state ^ INVERT;
        }
    }
    else
    {
        pressed = 0;
    }

    if( state != oldstate )
    {
        pp_write( state );
        oldstate = state;
    }

    return TRUE;
}

char * webcam_getimage( int *size )
{
    struct video_capability vid_caps;
    struct video_mbuf vid_buf;
    struct video_mmap vid_mmap;

    char *map = NULL;
    int len;
    static int frame = 0;
    int captured_frame;

    *size = 0;

    if (ioctl (fd_webcam, VIDIOCGCAP, &vid_caps) == -1)
    {
        printf("ioctl (VIDIOCGCAP) failed\n");
        return NULL;
    }
    printf( "V4L device %s %d channels %d audios %d < w < %d %d < h < %d\n",
             vid_caps.name, vid_caps.channels,
             vid_caps.audios, vid_caps.minwidth,  vid_caps.maxwidth,
             vid_caps.minheight, vid_caps.maxheight );
    if ( !(vid_caps.type & VID_TYPE_CAPTURE))
    {
        printf("cannot capture\n");
        return NULL;
    }

    if (ioctl (fd_webcam, VIDIOCGMBUF, &vid_buf) == -1)
    {
        // to do a normal read()
        map = malloc (WIDTH * HEIGHT * 3);
        len = read (fd_webcam, map, WIDTH * HEIGHT * 3);
        if (len <=  0)
        {
            free (map);
            return (NULL);
        }
        *size = 0;
        return (map);
    }

    map = mmap (0, vid_buf.size, PROT_READ|PROT_WRITE,MAP_SHARED,fd_webcam,0);
    if ((unsigned char *)-1 == (unsigned char *)map)
    {
        printf("mmap() failed \n");
        return (NULL);
    }

    captured_frame = frame;
    vid_mmap.frame = (frame + 1) % vid_buf.frames;

    vid_mmap.format = v_pict.palette;
    vid_mmap.width = WIDTH;
    vid_mmap.height = HEIGHT;
    //printf("VIDIOCMCAPTURE started\n");
    while (ioctl (fd_webcam, VIDIOCMCAPTURE, &vid_mmap) < 0)
    {
        printf("VIDIOCMCAPTURE failed, trying again\n");
        //munmap (map, vid_buf.size);
        //return (NULL);
    }
    //printf("VIDIOCSYNC started\n");
    while (ioctl (fd_webcam, VIDIOCSYNC, &frame) < 0 && (errno == EAGAIN || errno == EINTR));

    //{
    //    printf("VIDIOCSYNC failed \n");
    //    munmap (map, vid_buf.size);
    //    return (NULL);
    //}
    frame = vid_mmap.frame;
    //printf("VIDIOCSYNC finished %d bytes\n",vid_buf.size);
    // vid_buf.size is actually the size of 2 frame buffers in the driver
    *size = vid_buf.size;

fprintf(stderr, "ok\n");
    return map + vid_buf.offsets[captured_frame];
}

void webcam_capture( void )
{
    int size;
    char *image;
    if(!v_pict.palette)
    {
        // get palette info
        if(ioctl(fd_webcam, VIDIOCGPICT, &v_pict)==-1)
        {
            g_print(_("VIDIOCGPICT failed\n"));
            return;
        }
	printf("palette %i\n", v_pict.palette);
        if(v_pict.palette!=VIDEO_PALETTE_YUV420P
            && v_pict.palette!=VIDEO_PALETTE_RGB24)
        {
            g_print(_("Bad palette\n"));
            return;
        }
    }

    image = webcam_getimage(&size);

    if ( image )
    {
        if(v_pict.palette==VIDEO_PALETTE_YUV420P)
        {
            memcpy( image1, image, 640 * 480 );
        }
    }
    else
    {
        g_print(_("image capture failed\n"));
    }

    // then dont forget to release buffers
    if (size)
    {
        munmap(image, size);
    }
    else
    {
        if (image)
        {
            free(image);
        }
    }
}

int yfromx( int x, int a, int aa, int b, int bb )
{
    if(bb == aa) return -1;
    return bb+(x-b)*(a-b)/(bb-aa);
}

int xfromy( int y, int a, int aa, int b, int bb )
{
    if(a == b) return -1;
    return b+(y-bb)*(bb-aa)/(a-b);
}

void bresenham_line( guchar *image, guchar color, int a, int aa, int b, int bb )
{
#define PIXEL(X,Y) ((int)image[(Y)*640+(X)])
    int dX = abs(b-a);
    int dY = abs(bb-aa);

    int CurrentX = a;
    int CurrentY = aa;
 
    int Xincr, Yincr;
    if (a > b) { Xincr=-1; } else { Xincr=1; }
    if (aa > bb) { Yincr=-1; } else { Yincr=1; }

    if (dX >= dY)
    {
        int dPr     = dY<<1;   
        int dPru     = dPr - (dX<<1);
        int P         = dPr - dX;

        for (; dX>=0; dX--)
        {
            PIXEL(CurrentX, CurrentY) = color;
            if (P > 0)
            { 
                CurrentX+=Xincr;
                CurrentY+=Yincr; 
                P+=dPru;
            }
            else
            {
                CurrentX+=Xincr;
                P+=dPr;
            }
        }
    }
    else
    {
        int dPr     = dX<<1;   
        int dPru     = dPr - (dY<<1);
        int P         = dPr - dY;

        for (; dY>=0; dY--)
        {
            PIXEL(CurrentX, CurrentY) = color;
            if (P > 0)
            { 
                CurrentX+=Xincr; 
                CurrentY+=Yincr;
                P+=dPru;
            }
            else
            {
                CurrentY+=Yincr;
                P+=dPr;
            }
        }
    }
#undef PIXEL
}

unsigned char clip_bits( int x, int y )
{
    unsigned char b = 0;

    if( x < 0 ) b |= (1<<0);
    else if( x >= 640 ) b |= (1<<1);

    if( y < 0 ) b |= (1<<2);
    else if( y >= 480 ) b |= (1<<3);

    return b;
}

void draw_line( guchar *image, char color, int a, int aa, int b, int bb )
{
    int x2, y2;
    unsigned char b1, b2;

    b1 = clip_bits( a, aa );
    b2 = clip_bits( b, bb );

    if( b1 & b2 )
    {
        return;
    }

    if( (b1 | b2) == 0 )
    {
        bresenham_line( image, color, a, aa, b, bb );
        return;
    }

    if( b1 == 0 )
    {
        draw_line( image, color, b, bb, a, aa );
        return;
    }

    if( b1 & (1<<0) )
    {
        aa = bb - (b-0)*(bb-aa)/(b-a);
        a = 0;
    }
    else if( b1 & (1<<1) )
    {
        aa = bb - (b-639)*(bb-aa)/(b-a);
        a = 639;
    }
    else if( b1 & (1<<2) )
    {
        a = b - (bb-0)*(b-a)/(bb-aa);
        aa = 0;
    }
    else if( b1 & (1<<3) )
    {
        a = b - (bb-479)*(b-a)/(bb-aa);
        aa = 479;
    }

    draw_line( image, color, a, aa, b, bb );
}

void expand_border(guchar *dest)
{
    int x, y;

    dest_pixel(0,0) = dest_pixel(1,1);
    dest_pixel(0,479) = dest_pixel(1,478);
    dest_pixel(639,0) = dest_pixel(638,1);
    dest_pixel(639,479) = dest_pixel(638,478);

    for(x=1; x< 639; x++)
    {
        dest_pixel(x,0) = dest_pixel(x,1);
        dest_pixel(x,479) = dest_pixel(x,478);
    }

    for(y=1; y< 479; y++)
    {
        dest_pixel(0,y) = dest_pixel(1,y);
        dest_pixel(639,y) = dest_pixel(638,y);
    }
}

void smooth_mean(guchar *dest, guchar *source)
{
    int x, y;

    for(x=1; x<639; x++)
        for(y=1; y<479; y++)
            //dest_pixel(x,y) = (source_pixel(x,y) + source_pixel(x,y+1) + source_pixel(x+1,y+1) + source_pixel(x+1,y)) / 4;
            dest_pixel(x,y) = (source_pixel(x,y) + source_pixel(x,y+1) + source_pixel(x+1,y+1) + source_pixel(x+1,y) + source_pixel(x,y-1) + source_pixel(x+1,y-1) + source_pixel(x-1,y-1) + source_pixel(x-1,y+1) + source_pixel(x-1,y)) / 9;

    expand_border(dest);
}

guchar median3( guchar a, guchar b, guchar c )
{
    if( a <= b )
        if( b <= c )
            return b;
        else
            if( c <= a )
                return a;
            else
                return c;
    else
        if( c <= b )
            return b;
        else
            if( a <= c )
                return a;
            else
                return c;
}

void smooth_median(guchar *dest, guchar *source)
{
    int x, y, i, j;
    guchar a[9];

    for(x=1; x<639; x++)
    {
        for(y=1; y<479; y++)
        {
            a[0] = source_pixel(x-1,y-1);
            a[1] = source_pixel(x-1,y);
            a[2] = source_pixel(x-1,y+1);
            a[3] = source_pixel(x,y-1);
            a[4] = source_pixel(x,y);
            a[5] = source_pixel(x,y+1);
            a[6] = source_pixel(x+1,y-1);
            a[7] = source_pixel(x+1,y);
            a[8] = source_pixel(x+1,y+1);
            // bubble sort kicks ass !!
            for(i=0; i<5; i++) // pure optim: stop when a[4] is known
                for(j=i+1; j<9; j++)
                    if(a[i]>a[j])
                    {
                        register int k=a[i];
                        a[i]=a[j];
                        a[j]=k;
                    }

            dest[y*640+x] = a[4];
        }
    }

    expand_border(dest);
}

void edge_detection(guchar *dest, guchar *source)
{
    int x, y;
    guchar tmp1, tmp2;
    int foo;

    for(x=1; x<639; x++)
    {
        for(y=1; y<479; y++)
        {
            foo = 0;

            tmp1 = (source_pixel(x,y) + 256 - source_pixel(x+1,y)) / 2;
            tmp2 = (source_pixel(x,y) + 256 - source_pixel(x-1,y)) / 2;
            if( (tmp1 < 124 || tmp1 > 132)
                 && (tmp2 < 124 || tmp2 > 132) ) foo++;

            tmp1 = (source_pixel(x,y) + 256 - source_pixel(x+1,y+1)) / 2;
            tmp2 = (source_pixel(x,y) + 256 - source_pixel(x-1,y-1)) / 2;
            if( (tmp1 < 122 || tmp1 > 134)
                 && (tmp2 < 122 || tmp2 > 134) ) foo++;

            tmp1 = (source_pixel(x,y) + 256 - source_pixel(x,y+1)) / 2;
            tmp2 = (source_pixel(x,y) + 256 - source_pixel(x,y-1)) / 2;
            if( (tmp1 < 122 || tmp1 > 134)
                 && (tmp2 < 122 || tmp2 > 134) ) foo++;

            tmp1 = (source_pixel(x,y) + 256 - source_pixel(x-1,y+1)) / 2;
            tmp2 = (source_pixel(x,y) + 256 - source_pixel(x+1,y-1)) / 2;
            if( (tmp1 < 124 || tmp1 > 132)
                 && (tmp2 < 124 || tmp2 > 132) ) foo++;

            dest[y*640+x] = foo * 32;
        }
    }

    expand_border(dest);
}

void sobel(guchar *dest, guchar *source)
{
    int x, y;
    int tmp1, tmp2;

    for(x=1; x<639; x++)
    {
        for(y=1; y<479; y++)
        {
            tmp1 = ( source_pixel(x+1,y+1) + 2 * source_pixel(x,y+1) + source_pixel(x-1,y+1)
                     - source_pixel(x+1,y-1) - 2 * source_pixel(x,y-1) - source_pixel(x-1,y-1));
            if( tmp1 < 0 ) tmp1 = -tmp1;
            tmp2 = ( source_pixel(x+1,y+1) + 2 * source_pixel(x+1,y) + source_pixel(x+1,y-1)
                     - source_pixel(x-1,y+1) - 2 * source_pixel(x-1,y) - source_pixel(x-1,y-1));
            if( tmp2 < 0 ) tmp2 = -tmp2;

            if(tmp1 + tmp2 > 512)
                dest[y*640+x] = 255;
            else
                dest[y*640+x] = (tmp1 + tmp2) / 2;
        }
    }

    expand_border(dest);
}

void laplacien(guchar *dest, guchar *source)
{
    int x, y;
    int tmp;

    for(x=1; x<639; x++)
    {
        for(y=1; y<479; y++)
        {
            tmp = (int)( source_pixel(x+1,y) + source_pixel(x-1,y) + source_pixel(x,y+1) + source_pixel(x,y-1) - 4 * source_pixel(x,y)) / 16;
            dest[y*640+x] = tmp;
        }
    }

    expand_border(dest);
}

void threshold(guchar *source, guchar t)
{
    int x, y;

    for(x=0; x<640; x++)
        for(y=0; y<480; y++)
            if( source_pixel(x,y) > t )
                source_pixel(x,y) = 255;
            else
                source_pixel(x,y) = 0;
}

void halve(guchar *source)
{
    int x, y;

    for(x=0; x<640; x++)
        for(y=0; y<480; y++)
            source_pixel(x,y) >>= 1;
}

void draw_big( guchar *target, guchar *source, char *name )
{
    GtkWidget *wid;

    memcpy( target, source, 640 * 480 );

    gdk_threads_enter();
    wid = GTK_WIDGET(lookup_widget(window1,name));
    gtk_widget_queue_draw(wid);
    gdk_threads_leave();
}

void draw_mini( guchar *target, guchar *source, char *name )
{
    GtkWidget *wid;

    int x, y;
    for(x=0; x<320; x++)
        for(y=0; y<240; y++)
            target[y*320+x] = (source_pixel(2*x,2*y) + source_pixel(2*x+1,2*y) + source_pixel(2*x,2*y+1) + source_pixel(2*x+1,2*y+1)) / 4;

    gdk_threads_enter();
    wid = GTK_WIDGET(lookup_widget(window1,name));
    gtk_widget_queue_draw(wid);
    gdk_threads_leave();
}

void hough (/*struct image*/ guchar *source, int t)
{
    guchar *tmp;
    int w, w1, w2, w3, w4;
    int x, y, x2, y2, r;
    double conv = 3.1415926535/180.0;

    int stats = 0;

    /* Initialize a table of sine and cosine values */
    for (w=0; w < 180; w++)
    {
        sintable[w] = sin((double)(w*conv));
        costable[w] = cos((double)(w*conv));
    }

    /* Clear the Hough image */
    for (r = 0; r < 401; r++)
        for (w = 0; w < 180; w++)
             hough_space[w*401+r] = 0;

    /* Transform each pixel in X into Hough coordinate (r,w) if
       it has a value >= T. Increment all Hough pixels that correspond */
    for (y = 0; y < 480; y++)
    {
        tmp = source + 640 * y;

        for (x = 0; x < 640; x++)
        {
            if (tmp[x] >= t)
            {
                stats++;
                x2 = x - 320; y2 = y - 240;

                // My cool optimization : check 
                if( x2 * y2 > 0 )
                    w = atan( (double)x2 / (double)y2 ) / conv;
                else if( x2 * y2 < 0 )
                    w = 180 - atan( -(double)x2 / (double)y2 ) / conv;
                else
                    w = 180;

                if( x2 < 0 )
                {
                    w1 = w; w2 = 180; w3 = 0; w4 = w;
                }
                else
                {
                    w1 = 0; w2 = w; w3 = w; w4 = 180;
                }

                for (w = w1; w < w2; ++w)
                    hough_space[w*401+201+(int)(x2*costable[w]-y2*sintable[w])]++;

                for (w = w3; w < w4; ++w)
                    hough_space[w*401+200+(int)(x2*costable[w]-y2*sintable[w])]++;
            }
        }
    }
}

void hough_stats( void )
{
    int angles[45+2], *glop;
    int r, w, i, j;

    memset(angles, 0x00, sizeof(angles));
    memset(block, 0x00, sizeof(block));
    glop = angles + 1;

    for (r = 0; r < 401; r++)
    {
        for (w = 0; w < 180; w++)
        {
            if (hough_space[w*401+r] > 150 )
            {
                glop[w/4]++;
            }
        }
    }

#if 1
    for( w = 0; w < 45 ; w++ )
    {
        if( glop[w-1] == 0 && glop[w+1] == 0 && glop[w] <= 5 )
            glop[w] = 0;

        if( glop[w-1] != 0 && glop[w+1] != 0 && glop[w] == 0 )
            glop[w] = 1;

        if( glop[w-1] <= 5 && glop[w] <= glop[w+1] )
            glop[w+1] += glop[w];

        if( glop[w+1] <= 5 && glop[w] <= glop[w-1] )
            glop[w-1] += glop[w];
    }
#endif

    blocks = 0;

    for( w = 0; w < 45 ; w++ )
    {
        if( glop[w-1] == 0 && glop[w] != 0 )
        {
            block[blocks][0] = w * 4;
        }

        if( glop[w-1] != 0 && glop[w] == 0 )
        {
            block[blocks][1] = w * 4;
            blocks++;
        }

        block[blocks][2] += glop[w];
    }

    //printf("%i blocks found\n", blocks);

    // bubble sort kicks butt !!
    for(i=0; i<2; i++) // pure optim: stop when we know 2 values
        for(j=i+1; j<blocks; j++)
            if(block[i][2]<block[j][2])
            {
                register int k;
                k = block[i][0]; block[i][0] = block[j][0]; block[j][0] = k;
                k = block[i][1]; block[i][1] = block[j][1]; block[j][1] = k;
                k = block[i][2]; block[i][2] = block[j][2]; block[j][2] = k;
            }
}

void draw_hough( guchar *source )
{
    int r, w, x2, y2, b;
    double conv = 3.1415926535/180.0;
    GtkWidget *wid;

    for (w=0 ; w<180; w++)
        for (r = 0; r < 401; r++)
            hough_camera[w*401+r] = hough_space[w*401+r] / 2;

    gdk_threads_enter();
    wid = GTK_WIDGET(lookup_widget(window1,"houghcamera"));
    gtk_widget_queue_draw(wid);
    gdk_threads_leave();

    //for( b = 0; b < blocks; b++ ) /* draw all blocks */
    for( b = 0; b < 2; b++ ) /* Only draw the first 2 blocks */
    {
        for (w = block[b][0]; w < block[b][1]+1; w++)
        {
            for (r = 0; r < 401; r++)
            {
                if (hough_space[w*401+r] > 120 )
                {
                    double c = cos(w*conv), s = sin(w*conv);
                    x2 = 320 + (r - 200) * c;
                    y2 = 240 - (r - 200) * s;

                    // draw the control line
                    //bresenham_line( source, 0xbf, 320, 240, x2, y2 );

                    // draw the real line
                    draw_line( source, 0xff,
                               x2 - 600*s, y2 - 600*c, x2 + 600*s, y2 + 600*c);
                }
            }
        }
    }
}

void rabote(guchar *dest, guchar *source)
{
    int x, y;
    int foo;

    for(x=1; x<639; x++)
        for(y=1; y<479; y++)
        {
            foo = 0;

            if( source_pixel(x,y) &&
                 ( source_pixel(x+1,y) || source_pixel(x-1,y)
                    || source_pixel(x,y+1) || source_pixel(x,y-1) ) )
            {
                dest[y*640+x] = source_pixel(x,y);
            }
            else
            {
                dest[y*640+x] = 0;
            }
        }
}

gint webcam_callback( gpointer data )
{
#define SWAP() { guchar *tmp = src ; src = dst ; dst = tmp; }
    guchar *src = image1, *dst = image2;

    do
    {
        webcam_capture();

        draw_big( main_camera, src, "fullcamera" );

        smooth_median(dst, src); SWAP();
#if 0
        smooth_mean(dst, src); SWAP();
#endif

        draw_mini(camera_1, src, "camera_1" );

        edge_detection(dst, src); SWAP();
#if 0
        sobel(dst, src); SWAP();
        laplacien(dst, src); SWAP();
#endif
        //threshold( src, 50 );
        //rabote(dst, src); SWAP();
        //draw_big( final_camera, src, "finalcamera" );

        draw_mini(camera_3, src, "camera_3" );

        hough( src, 60 );
        hough_stats();
        SWAP();
        halve( src );
        draw_hough( src );

        draw_mini(camera_2, src, "camera_2" );

        draw_big( final_camera, src, "finalcamera" );

        SWAP();

        if( !webcam_thread ) break;

        sleep(1);

    } while( webcam_thread );

    return NULL;
}

int
main (int argc, char *argv[])
{
    int i;
    long int *ptr;
    pthread_t tid;

    state = 0;
    oldstate = 1;
    webcam_thread = 1;

#ifdef ENABLE_NLS
    bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
    textdomain (PACKAGE);
#endif

    g_thread_init (NULL);

    gnome_init ("plop", VERSION, argc, argv);

    // do the inits for the rgb buffers
    gdk_rgb_init();
    gtk_widget_push_visual(gdk_rgb_get_visual());
    gtk_widget_push_colormap(gdk_rgb_get_cmap());
    window1 = create_app1();
    gtk_widget_pop_visual();
    gtk_widget_pop_colormap();

    gtk_widget_show(window1);

    // get parallel port
    fd_parport = open("/dev/parport0", O_RDWR);
    ioctl(fd_parport,PPCLAIM);
    pp_write(0);

    // get webcam
    fd_webcam = open("/dev/video0", O_RDWR);
    v_pict.palette = 0;

    // main
    gtk_timeout_add( 20, pp_callback, NULL );
    //gtk_timeout_add( 2000, webcam_callback, NULL );

    pthread_create(&tid, NULL, webcam_callback, NULL );

    gdk_threads_enter();
    gtk_main ();
    gdk_threads_leave();

    webcam_thread = 0;
    pthread_join(tid, NULL);

    // release webcam
    close(fd_webcam);

    // release parport
    pp_write(0);
    ioctl(fd_parport,PPRELEASE);
    close(fd_parport);

    return 0;
}

